﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new GarmentFactoryDatabase();

            return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.Select(x => x.GetViewModel)
                .ToList();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new GarmentFactoryDatabase();
			if (model.Id.HasValue)
			{
				return context.Orders
					.Include(x => x.Textile)
					.Include(x => x.Client)
					.Include(x => x.Implementer)
					.Where(x => x.Id == model.Id)
					.Select(x => x.GetViewModel)
					.ToList();
			}

			else if (model.DateFrom.HasValue)
            {
                return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                .Select(x => x.GetViewModel)
                .ToList();
            }

            else if (model.ClientId.HasValue)
            {
                return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.Where(x => x.ClientId == model.ClientId)
                .Select(x => x.GetViewModel)
                .ToList();
            }

            else if (model.ImplementerId.HasValue)
            {
                 return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.Where(x => x.ImplementerId == model.ImplementerId)
                .Select(x => x.GetViewModel)
                .ToList();
            }

            else if (model.Status.HasValue)
            {
                 return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.Where(x => x.Status == model.Status)
                .Select(x => x.GetViewModel)
                .ToList();
            }
            return new();
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue && (!model.ImplementerId.HasValue || !model.Status.HasValue))
            {
                return null;
            }

            using var context = new GarmentFactoryDatabase();

            return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id) ||
                (model.ImplementerId.HasValue && x.ImplementerId == model.ImplementerId && x.Status == model.Status))
                ?.GetViewModel;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }

            using var context = new GarmentFactoryDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();

            return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
                .Include (x => x.Implementer)
                .FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel;
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new GarmentFactoryDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);

            if (order == null)
            {
                return null;
            }

            order.Update(model);
            context.SaveChanges();

            return context.Orders
                .Include(x => x.Textile)
                .Include(x => x.Client)
				.Include(x => x.Implementer)
				.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new GarmentFactoryDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);

            if (element != null)
            {
                var delElement = context.Orders.Include(x => x.Textile).Include(x => x.Client).Include(x => x.Implementer).FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
				context.Orders.Remove(element);
                context.SaveChanges();
                return delElement;
            }
            return null;
        }
    }
}
