﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;
        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<OrderViewModel> GetFullList() => source.Orders.Select(x => AddTextileName(AddClientFIO(AddImplementerFIO(x.GetViewModel)))).ToList();
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (model.Id.HasValue)
            {
                return source.Orders.Where(x => x.Id == model.Id).Select(x => AddTextileName(AddClientFIO(AddImplementerFIO(x.GetViewModel)))).ToList();
            }
            else if (model.DateFrom.HasValue)
            {
                return source.Orders.Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo).Select(x => AddTextileName(AddClientFIO(AddImplementerFIO(x.GetViewModel)))).ToList();
            }
            else if (model.ClientId.HasValue)
            {
                return source.Orders.Where(x => x.ClientId == model.ClientId).Select(x => AddTextileName(AddClientFIO(AddImplementerFIO(x.GetViewModel)))).ToList();
            }
			else if (model.ImplementerId.HasValue)
            {
				return source.Orders.Where(x => x.ImplementerId == model.ImplementerId).Select(x => AddTextileName(AddClientFIO(AddImplementerFIO(x.GetViewModel)))).ToList();

			}
			return new();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            return AddTextileName(AddClientFIO(AddImplementerFIO(source.Orders.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel)));
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return AddTextileName(AddClientFIO(AddImplementerFIO(newOrder.GetViewModel)));
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            source.SaveOrders();
            return AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel)));
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order != null)
            {
                source.Orders.Remove(order);
                source.SaveOrders();
                return AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel)));
            }
            return null;
        }

		private OrderViewModel? AddImplementerFIO(OrderViewModel? model)
		{
			if (model == null)
			{
				return null;
			}
			var implementer = source.Implementers.FirstOrDefault(x => x.Id == model.ImplementerId);
			if (implementer != null)
			{
                model.ImplementerFIO = implementer.ImplementerFIO;
			}
			return model;
		}

		private OrderViewModel? AddClientFIO(OrderViewModel? model)
        {
            if (model == null)
            {
                return null;
            }
            var client = source.Clients.FirstOrDefault(x => x.Id == model.ClientId);
            if (client != null)
            {
                model.ClientFIO = client.ClientFIO;
            }
            return model;
        }


        private OrderViewModel? AddTextileName(OrderViewModel? model)
        {
            if (model == null)
            {
                return null;
            }
            var textile = source.Textiles.FirstOrDefault(x => x.Id == model.TextileId);
            model.TextileName = textile?.TextileName;
            return model;
        }
    }
}
