﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryFileImplement.Implements
{
    public class TextileStorage : ITextileStorage
    {
        private readonly DataFileSingleton source;
        public TextileStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public List<TextileViewModel> GetFullList()
        {
            return source.Textiles.Select(x => x.GetViewModel).ToList();
        }

        public List<TextileViewModel> GetFilteredList(TextileSearchModel model)
        {
            if (string.IsNullOrEmpty(model.TextileName))
            {
                return new();
            }
            return source.Textiles.Where(x => x.TextileName.Contains(model.TextileName)).Select(x => x.GetViewModel).ToList();
        }
        public TextileViewModel? GetElement(TextileSearchModel model)
        {
            if (string.IsNullOrEmpty(model.TextileName) && !model.Id.HasValue)
            {
                return null;
            }
            return source.Textiles.FirstOrDefault(x =>
                (!string.IsNullOrEmpty(model.TextileName) && x.TextileName == model.TextileName) ||
                (model.Id.HasValue && x.Id == model.Id))
                ?.GetViewModel;
        }
        public TextileViewModel? Insert(TextileBindingModel model)
        {
            model.Id = source.Textiles.Count > 0 ? source.Textiles.Max(x => x.Id) + 1 : 1;
            var newTextile = Textile.Create(model);
            if (newTextile == null)
            {
                return null;
            }
            source.Textiles.Add(newTextile);
            source.SaveTextiles();
            return newTextile.GetViewModel;
        }
        public TextileViewModel? Update(TextileBindingModel model)
        {
            var Textile = source.Textiles.FirstOrDefault(x => x.Id == model.Id);
            if (Textile == null)
            {
                return null;
            }
            Textile.Update(model);
            source.SaveTextiles();
            return Textile.GetViewModel;
        }
        public TextileViewModel? Delete(TextileBindingModel model)
        {
            var textile = source.Textiles.FirstOrDefault(x => x.Id == model.Id);
            if (textile != null)
            {
                source.Textiles.Remove(textile);
                source.SaveTextiles();
                return textile.GetViewModel;
            }
            return null;
        }
    }
}
