﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryDataModels.Enums;
using GarmentFactoryDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace GarmentFactoryFileImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int ClientId { get; private set; }
		public int? ImplementerId { get; set; }
		public int TextileId { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateComplete { get; private set; }
        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                ClientId = model.ClientId,
				ImplementerId = model.ImplementerId,
				TextileId = model.TextileId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateComplete = model.DateComplete
            };
        }
        public static Order? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            string dateComplete = element.Element("DateComplete")!.Value;
            return new Order()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                TextileId = Convert.ToInt32(element.Element("TextileId")!.Value),
				ImplementerId = Convert.ToInt32(element.Element("ImplementerId")!.Value),
				Count = Convert.ToInt32(element.Element("Count")!.Value),
                Sum = Convert.ToDouble(element.Element("Sum")!.Value),
                Status = (OrderStatus)(Enum.Parse(typeof(OrderStatus), element.Element("Status")!.Value)),
                DateCreate = Convert.ToDateTime(element.Element("DateCreate")!.Value),
                DateComplete = (dateComplete == "" || dateComplete is null) ? Convert.ToDateTime(null) : Convert.ToDateTime(dateComplete)
            };

        }
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            if (model.Status == OrderStatus.Выдан) {
                DateComplete = model.DateComplete;
            } 
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ClientId = ClientId,
            TextileId = TextileId,
			ImplementerId = ImplementerId,
			Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateComplete = DateComplete
        };
        public XElement GetXElement => new("Order",
           new XAttribute("Id", Id),
           new XElement("ClientId", ClientId.ToString()),
           new XElement("TextileId", TextileId.ToString()),
		   new XElement("ImplementerId", ImplementerId),
		   new XElement("Count", Count.ToString()),
           new XElement("Sum", Sum.ToString()),
           new XElement("Status", Status.ToString()),
           new XElement("DateCreate", DateCreate.ToString()),
           new XElement("DateComplete", DateComplete.ToString()));
    }
}
