﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.SearchModels;
using GarmentFactoryContracts.StoragesContracts;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryListImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataListSingleton _source;
        public OrderStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<OrderViewModel> GetFullList()
        {
            var result = new List<OrderViewModel>();

            foreach (var order in _source.Orders)
            {
                result.Add(AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel))));
            }

            return result;
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            var result = new List<OrderViewModel>();

			if (model.Id.HasValue)
			{
				foreach (var order in _source.Orders)
				{
					if (order.Id == model.Id)
					{
						result.Add(AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel))));
					}
				}
			}
			else if (model.DateFrom.HasValue)
            {
                foreach (var order in _source.Orders)
                {
                    if (order.DateCreate >= model.DateFrom && order.DateCreate <= model.DateTo)
                    {
						result.Add(AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel))));
					}
                }
            }
            else if (model.ClientId.HasValue)
            {
                foreach (var order in _source.Orders)
                {
                    if (order.ClientId == model.ClientId)
                    {
						result.Add(AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel))));
					}
                }
            }
			else if (model.ImplementerId.HasValue)
			{
				foreach (var order in _source.Orders)
				{
					if (order.ImplementerId == model.ImplementerId)
					{
						result.Add(AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel))));
					}
				}
			}
			return result;
        }


        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }

            foreach (var order in _source.Orders)
            {
                if (model.Id.HasValue && order.Id == model.Id)
                {
                    return AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel)));
                }
            }

            return null;
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = 1;

            foreach (var order in _source.Orders)
            {
                if (model.Id <= order.Id)
                {
                    model.Id = order.Id + 1;
                }
            }

            var newOrder = Order.Create(model);

            if (newOrder == null)
            {
                return null;
            }

            _source.Orders.Add(newOrder);

			return AddTextileName(AddClientFIO(AddImplementerFIO(newOrder.GetViewModel)));
		}

        public OrderViewModel? Update(OrderBindingModel model)
        {
            foreach (var order in _source.Orders)
            {
                if (order.Id == model.Id)
                {
                    order.Update(model);
                    return AddTextileName(AddClientFIO(AddImplementerFIO(order.GetViewModel)));
				}
            }

            return null;
        }

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            for (int i = 0; i < _source.Orders.Count; ++i)
            {
                if (_source.Orders[i].Id == model.Id)
                {
                    var element = _source.Orders[i];
                    _source.Orders.RemoveAt(i);
                    return AddTextileName(AddClientFIO(AddImplementerFIO(element.GetViewModel)));
				}
            }
            return null;
        }

		private OrderViewModel AddImplementerFIO(OrderViewModel model)
		{
			var implementer = _source.Implementers.FirstOrDefault(x => x.Id == model.ClientId);
			if (implementer != null)
			{
				model.ImplementerFIO = implementer.ImplementerFIO;
			}
			return model;
		}

		private OrderViewModel AddClientFIO(OrderViewModel model)
        {
            var client = _source.Clients.FirstOrDefault(x => x.Id == model.ClientId);
            if (client != null)
            {
                model.ClientFIO = client.ClientFIO;
            }
            return model;
        }

        private OrderViewModel AddTextileName(OrderViewModel model)
        {
            foreach (var textile in _source.Textiles)
            {
                if (textile.Id == model.TextileId)
                {
                    model.TextileName = textile.TextileName;
                    return model;
                }
            }
            return model;
        }
    }
}
